# App::hopen::T::MSCL - Microsoft cl.exe toolset
package App::hopen::T::MSCL;
use Data::Hopen;
use strict; use warnings;
use Data::Hopen::Base;

our $VERSION = '0.000012'; # TRIAL

#use parent 'Data::Hopen::TODO';
#use Class::Tiny qw(TODO);

# Docs {{{1

=head1 NAME

Data::Hopen::T::MSCL - Microsoft cl.exe toolset for hopen

=head1 SYNOPSIS

This toolset supports any compiler and linker that will accept C<cl.exe>
options.

=cut

# }}}1

1;
__END__
# vi: set fdm=marker: #
